@@ -1,10 +1,10 @@
 from django.core.exceptions import ValidationError
 from django.utils import timezone
 
-from caluma.caluma_core.events import send_event
+from caluma.caluma_form.models import Document
 
-from ..caluma_form.models import Document
-from . import api, events, models, utils, validators
+from . import api, models, utils, validators
+from .events import send_event_with_deprecations
 
 
 class StartCaseLogic:
@@ -35,7 +35,16 @@ class StartCaseLogic:
         return data
 
     @staticmethod
-    def pre_start(validated_data, user):
+    def pre_start(validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_create_case",
+            sender="pre_create_case",
+            case=None,
+            validated_data=validated_data,
+            user=user,
+            context=context,
+        )
+
         parent_work_item = validated_data.get("parent_work_item")
         validated_data["status"] = models.Case.STATUS_RUNNING
 
@@ -66,16 +75,17 @@ class StartCaseLogic:
 
         work_items = utils.bulk_create_work_items(tasks, case, user, None, context)
 
-        send_event(
-            events.created_case,
+        send_event_with_deprecations(
+            "post_create_case",
             sender="case_post_create",
             case=case,
             user=user,
             context=context,
         )
+
         for work_item in work_items:  # pragma: no cover
-            send_event(
-                events.created_work_item,
+            send_event_with_deprecations(
+                "post_create_work_item",
                 sender="case_post_create",
                 work_item=work_item,
                 user=user,
@@ -113,7 +123,18 @@ class CompleteWorkItemLogic:
         )
 
     @staticmethod
-    def pre_complete(work_item, validated_data, user, context=None):
+    def pre_complete(
+        work_item,
+        validated_data,
+        user,
+        context=None,
+        event="pre_complete_work_item",
+        sender="pre_complete_work_item",
+    ):
+        send_event_with_deprecations(
+            event, sender=sender, work_item=work_item, user=user, context=context
+        )
+
         validated_data["status"] = models.WorkItem.STATUS_COMPLETED
         validated_data["closed_at"] = timezone.now()
         validated_data["closed_by_user"] = user.username
@@ -122,9 +143,19 @@ class CompleteWorkItemLogic:
         return validated_data
 
     @staticmethod
-    def post_complete(work_item, user, context=None):
+    def post_complete(
+        work_item,
+        user,
+        context=None,
+        event="post_complete_work_item",
+        sender="post_complete_work_item",
+    ):
         case = work_item.case
 
+        send_event_with_deprecations(
+            event, sender=sender, work_item=work_item, user=user, context=context
+        )
+
         if not CompleteWorkItemLogic._can_continue(work_item, work_item.task):
             return work_item
 
@@ -153,22 +184,14 @@ class CompleteWorkItemLogic:
                 )
 
                 for created_work_item in created_work_items:  # pragma: no cover
-                    send_event(
-                        events.created_work_item,
-                        sender="post_complete_work_item",
+                    send_event_with_deprecations(
+                        "post_create_work_item",
+                        sender=sender,
                         work_item=created_work_item,
                         user=user,
                         context=context,
                     )
 
-        send_event(
-            events.completed_work_item,
-            sender="post_complete_work_item",
-            work_item=work_item,
-            user=user,
-            context=context,
-        )
-
         if (
             not next_tasks.exists()
             and not work_item.case.work_items.filter(
@@ -182,9 +205,9 @@ class CompleteWorkItemLogic:
             case.closed_by_group = user.group
             case.save()
 
-            send_event(
-                events.completed_case,
-                sender="post_complete_work_item",
+            send_event_with_deprecations(
+                "post_complete_case",
+                sender=sender,
                 case=case,
                 user=user,
                 context=context,
@@ -205,7 +228,12 @@ class SkipWorkItemLogic:
     @staticmethod
     def pre_skip(work_item, validated_data, user, context=None):
         validated_data = CompleteWorkItemLogic.pre_complete(
-            work_item, validated_data, user, context
+            work_item,
+            validated_data,
+            user,
+            context,
+            event="pre_skip_work_item",
+            sender="pre_skip_work_item",
         )
 
         validated_data["status"] = models.WorkItem.STATUS_SKIPPED
@@ -220,14 +248,12 @@ class SkipWorkItemLogic:
 
     @staticmethod
     def post_skip(work_item, user, context=None):
-        work_item = CompleteWorkItemLogic.post_complete(work_item, user, context)
-
-        send_event(
-            events.skipped_work_item,
+        work_item = CompleteWorkItemLogic.post_complete(
+            work_item,
+            user,
+            context,
+            event="post_skip_work_item",
             sender="post_skip_work_item",
-            work_item=work_item,
-            user=user,
-            context=context,
         )
 
         return work_item
@@ -254,6 +280,14 @@ class CancelCaseLogic:
 
     @staticmethod
     def pre_cancel(case, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_cancel_case",
+            sender="pre_cancel_case",
+            case=case,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.Case.STATUS_CANCELED
         validated_data["closed_at"] = timezone.now()
         validated_data["closed_by_user"] = user.username
@@ -268,18 +302,8 @@ class CancelCaseLogic:
 
     @staticmethod
     def post_cancel(case, user, context=None):
-        # TODO: remove in the next major release since `events.cancelled_case`
-        # is deprecated in favor of `events.canceled_case`
-        send_event(
-            events.cancelled_case,
-            sender="post_cancel_case",
-            case=case,
-            user=user,
-            context=context,
-        )
-
-        send_event(
-            events.canceled_case,
+        send_event_with_deprecations(
+            "post_cancel_case",
             sender="post_cancel_case",
             case=case,
             user=user,
@@ -307,6 +331,14 @@ class SuspendCaseLogic:
 
     @staticmethod
     def pre_suspend(case, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_suspend_case",
+            sender="pre_suspend_case",
+            case=case,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.Case.STATUS_SUSPENDED
 
         for work_item in case.work_items.filter(status=models.WorkItem.STATUS_READY):
@@ -316,8 +348,8 @@ class SuspendCaseLogic:
 
     @staticmethod
     def post_suspend(case, user, context=None):
-        send_event(
-            events.suspended_case,
+        send_event_with_deprecations(
+            "post_suspend_case",
             sender="post_suspend_case",
             case=case,
             user=user,
@@ -345,6 +377,14 @@ class ResumeCaseLogic:
 
     @staticmethod
     def pre_resume(case, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_resume_case",
+            sender="pre_resume_case",
+            case=case,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.Case.STATUS_RUNNING
 
         for work_item in case.work_items.filter(
@@ -356,8 +396,8 @@ class ResumeCaseLogic:
 
     @staticmethod
     def post_resume(case, user, context=None):
-        send_event(
-            events.resumed_case,
+        send_event_with_deprecations(
+            "post_resume_case",
             sender="post_resume_case",
             case=case,
             user=user,
@@ -378,6 +418,14 @@ class CancelWorkItemLogic:
 
     @staticmethod
     def pre_cancel(work_item, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_cancel_work_item",
+            sender="pre_cancel_work_item",
+            work_item=work_item,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.WorkItem.STATUS_CANCELED
         validated_data["closed_at"] = timezone.now()
         validated_data["closed_by_user"] = user.username
@@ -393,19 +441,8 @@ class CancelWorkItemLogic:
 
     @staticmethod
     def post_cancel(work_item, user, context=None):
-        # TODO: remove in the next major release since
-        # `events.cancelled_work_item` is deprecated in favor of
-        # `events.canceled_work_item`
-        send_event(
-            events.cancelled_work_item,
-            sender="post_cancel_work_item",
-            work_item=work_item,
-            user=user,
-            context=context,
-        )
-
-        send_event(
-            events.canceled_work_item,
+        send_event_with_deprecations(
+            "post_cancel_work_item",
             sender="post_cancel_work_item",
             work_item=work_item,
             user=user,
@@ -423,6 +460,14 @@ class SuspendWorkItemLogic:
 
     @staticmethod
     def pre_suspend(work_item, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_suspend_work_item",
+            sender="pre_suspend_work_item",
+            work_item=work_item,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.WorkItem.STATUS_SUSPENDED
 
         if (
@@ -435,8 +480,8 @@ class SuspendWorkItemLogic:
 
     @staticmethod
     def post_suspend(work_item, user, context=None):
-        send_event(
-            events.suspended_work_item,
+        send_event_with_deprecations(
+            "post_suspend_work_item",
             sender="post_suspend_work_item",
             work_item=work_item,
             user=user,
@@ -454,6 +499,14 @@ class ResumeWorkItemLogic:
 
     @staticmethod
     def pre_resume(work_item, validated_data, user, context=None):
+        send_event_with_deprecations(
+            "pre_resume_work_item",
+            sender="pre_resume_work_item",
+            work_item=work_item,
+            user=user,
+            context=context,
+        )
+
         validated_data["status"] = models.WorkItem.STATUS_READY
 
         if (
@@ -466,8 +519,8 @@ class ResumeWorkItemLogic:
 
     @staticmethod
     def post_resume(work_item, user, context=None):
-        send_event(
-            events.resumed_work_item,
+        send_event_with_deprecations(
+            "post_resume_work_item",
             sender="post_resume_work_item",
             work_item=work_item,
             user=user,
