@@ -204,14 +204,20 @@ class CaseSerializer(SendEventSerializerMixin, ContextModelSerializer):
 class SaveCaseSerializer(CaseSerializer):
     @transaction.atomic
     def create(self, validated_data):
+        self.send_event(
+            events.pre_create_case, case=None, validated_data=validated_data
+        )
         instance = super().create(validated_data)
-        self.send_event(events.created_case)
+        self.send_event(events.post_create_case, case=instance)
         return instance
 
     @transaction.atomic
     def update(self, instance, validated_data):
+        self.send_event(
+            events.pre_create_case, case=instance, validated_data=validated_data
+        )
         instance = super().update(instance, validated_data)
-        self.send_event(events.created_case, case=instance)
+        self.send_event(events.post_create_case, case=instance)
         return instance
 
     class Meta(CaseSerializer.Meta):
@@ -423,8 +429,13 @@ class SaveWorkItemSerializer(SendEventSerializerMixin, ContextModelSerializer):
 
     @transaction.atomic
     def update(self, instance, validated_data):
+        self.send_event(
+            events.pre_create_work_item,
+            work_item=instance,
+            validated_data=validated_data,
+        )
         instance = super().update(instance, validated_data)
-        self.send_event(events.created_work_item, work_item=instance)
+        self.send_event(events.post_create_work_item, work_item=instance)
         return instance
 
     class Meta:
@@ -498,8 +509,11 @@ class CreateWorkItemSerializer(SendEventSerializerMixin, ContextModelSerializer)
 
     @transaction.atomic
     def create(self, validated_data):
+        self.send_event(
+            events.pre_create_work_item, work_item=None, validated_data=validated_data
+        )
         instance = super().create(validated_data)
-        self.send_event(events.created_work_item, work_item=instance)
+        self.send_event(events.post_create_work_item, work_item=instance)
         return instance
 
     class Meta:
