@@ -87,15 +87,49 @@ def test_query_all_work_items_filter_groups(
 
 @pytest.mark.parametrize("task__type,task__form", [(models.Task.TYPE_SIMPLE, None)])
 @pytest.mark.parametrize(
-    "work_item__status,case__status,success",
+    "work_item__status,case__status,child_case_status,error",
     [
-        (models.WorkItem.STATUS_READY, models.Case.STATUS_COMPLETED, True),
-        (models.WorkItem.STATUS_COMPLETED, models.Case.STATUS_COMPLETED, False),
-        (models.WorkItem.STATUS_READY, models.Case.STATUS_RUNNING, False),
+        (models.WorkItem.STATUS_READY, models.Case.STATUS_RUNNING, None, None),
+        (
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+            models.Case.STATUS_COMPLETED,
+            None,
+        ),
+        (
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+            models.Case.STATUS_COMPLETED,
+            None,
+        ),
+        (
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_COMPLETED,
+            None,
+            "Only work items of running cases can be completed.",
+        ),
+        (
+            models.WorkItem.STATUS_COMPLETED,
+            models.Case.STATUS_RUNNING,
+            None,
+            "Only ready work items can be completed.",
+        ),
+        (
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+            models.Case.STATUS_RUNNING,
+            "Work item can only be completed when child case is in a finish state.",
+        ),
     ],
 )
 def test_complete_work_item_last(
-    db, snapshot, work_item, success, admin_schema_executor
+    db,
+    snapshot,
+    work_item,
+    case_factory,
+    child_case_status,
+    error,
+    admin_schema_executor,
 ):
     query = """
         mutation CompleteWorkItem($input: CompleteWorkItemInput!) {
@@ -113,12 +147,16 @@ def test_complete_work_item_last(
           }
         }
     """
+    work_item.child_case = child_case_status and case_factory(status=child_case_status)
+    work_item.save()
 
     inp = {"input": {"id": work_item.pk}}
     result = admin_schema_executor(query, variable_values=inp)
 
-    assert not bool(result.errors) == success
-    if success:
+    assert bool(result.errors) == bool(error)
+    if error:
+        assert error in str(result.errors[0])
+    else:
         snapshot.assert_match(result.data)
 
 
@@ -351,19 +389,23 @@ def test_complete_multiple_instance_task_form_work_item_next(
     ],
 )
 @pytest.mark.parametrize(
-    "group_jexl",
+    "group_jexl,is_multiple_instance",
     [
-        '["some-group"]|groups',
-        "info.prev_work_item.controlling_groups",
-        "info.work_item.created_by_group",
-        "info.case.created_by_group",
-        "[info.case.created_by_group, info.work_item.created_by_group]|groups",
-        "[info.case.created_by_group, info.work_item.created_by_group]",
+        ('["some-group"]|groups', False),
+        ("info.prev_work_item.controlling_groups", False),
+        ("info.work_item.created_by_group", False),
+        ("info.case.created_by_group", False),
+        ("[info.case.created_by_group, info.work_item.created_by_group]|groups", False),
+        ("[info.case.created_by_group, info.work_item.created_by_group]", False),
+        ("[info.case.created_by_group, info.work_item.created_by_group]", True),
+        ("[]", False),
+        ("[]", True),
     ],
 )
 def test_complete_work_item_with_next(
     db,
     group_jexl,
+    is_multiple_instance,
     snapshot,
     work_item,
     task,
@@ -390,6 +432,7 @@ def test_complete_work_item_with_next(
         form=None,
         address_groups=group_jexl,
         control_groups=group_jexl,
+        is_multiple_instance=is_multiple_instance,
     )
     task_flow = task_flow_factory(task=task, workflow=workflow)
     task_flow.flow.next = f"'{task_next.slug}'|task"
@@ -875,6 +918,46 @@ def test_query_all_work_items_filter_case_meta_value(
     assert len(result.data["allWorkItems"]["edges"]) == len_results
 
 
+@pytest.mark.parametrize("value,len_results", [(1, 3), (2, 0), (3, 0), (4, 1)])
+def test_query_all_work_items_filter_root_case_meta_value(
+    db, work_item_factory, case_factory, schema_executor, value, len_results
+):
+    case = case_factory(meta={"testkey": 1})
+    child_case = case_factory(meta={"testkey": 2}, family=case)
+    child_child_case = case_factory(meta={"testkey": 3}, family=case)
+    unrelated_case = case_factory(meta={"testkey": 4})
+
+    # create a tree of case - work_item - child_case - workitem - child_child_case - work_item
+    work_item_factory(case=case, child_case=child_case)
+    work_item_factory(case=child_case, child_case=child_child_case)
+    work_item_factory(case=child_child_case)
+    work_item_factory(case=unrelated_case)
+
+    query = """
+        query WorkItems($root_case_meta_value: [JSONValueFilterType!]) {
+          allWorkItems(rootCaseMetaValue: $root_case_meta_value) {
+            totalCount
+            edges {
+              node {
+                case {
+                  meta
+                }
+              }
+            }
+          }
+        }
+    """
+
+    variables = {"key": "testkey", "value": value}
+
+    result = schema_executor(
+        query, variable_values={"root_case_meta_value": [variables]}
+    )
+
+    assert not result.errors
+    assert len(result.data["allWorkItems"]["edges"]) == len_results
+
+
 @pytest.mark.parametrize(
     "work_item__status,work_item__child_case,case__status,task__type,case__document",
     [
@@ -923,12 +1006,34 @@ def test_skip_task_form_work_item(
 
 
 @pytest.mark.parametrize(
-    "work_item_status,success",
+    "work_item_status,case__status,error",
     [
-        (models.WorkItem.STATUS_COMPLETED, False),
-        (models.WorkItem.STATUS_CANCELED, False),
-        (models.WorkItem.STATUS_SKIPPED, False),
-        (models.WorkItem.STATUS_READY, True),
+        (
+            models.WorkItem.STATUS_COMPLETED,
+            models.Case.STATUS_RUNNING,
+            "Only ready work items can be skipped.",
+        ),
+        (
+            models.WorkItem.STATUS_CANCELED,
+            models.Case.STATUS_RUNNING,
+            "Only ready work items can be skipped.",
+        ),
+        (
+            models.WorkItem.STATUS_SKIPPED,
+            models.Case.STATUS_RUNNING,
+            "Only ready work items can be skipped.",
+        ),
+        (
+            models.WorkItem.STATUS_SUSPENDED,
+            models.Case.STATUS_RUNNING,
+            "Only ready work items can be skipped.",
+        ),
+        (
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_COMPLETED,
+            "Only work items of running cases can be skipped.",
+        ),
+        (models.WorkItem.STATUS_READY, models.Case.STATUS_RUNNING, None),
     ],
 )
 @pytest.mark.parametrize("question__type,answer__value", [(Question.TYPE_INTEGER, 1)])
@@ -936,15 +1041,18 @@ def test_skip_multiple_instance_task_form_work_item(
     db,
     task_factory,
     work_item_factory,
+    case,
     answer,
     form_question,
     work_item_status,
-    success,
+    error,
     schema_executor,
 ):
     task = task_factory(is_multiple_instance=True)
-    work_item_1 = work_item_factory(task=task, child_case=None, status=work_item_status)
-    work_item_2 = work_item_factory(task=task, child_case=None, case=work_item_1.case)
+    work_item_1 = work_item_factory(
+        task=task, case=case, child_case=None, status=work_item_status
+    )
+    work_item_2 = work_item_factory(task=task, case=case, child_case=None)
     query = """
         mutation SkipWorkItem($input: SkipWorkItemInput!) {
           skipWorkItem(input: $input) {
@@ -962,10 +1070,10 @@ def test_skip_multiple_instance_task_form_work_item(
     inp = {"input": {"id": work_item_1.pk}}
     result = schema_executor(query, variable_values=inp)
 
-    assert bool(result.errors) != success
+    assert bool(result.errors) == bool(error)
 
-    if not success:
-        assert "Only READY work items can be skipped" in str(result.errors[0])
+    if error:
+        assert error in str(result.errors[0])
 
     else:
 
@@ -990,8 +1098,8 @@ def test_skip_multiple_instance_task_form_work_item(
 
 @pytest.mark.parametrize("task__type,task__form", [(models.Task.TYPE_SIMPLE, None)])
 @pytest.mark.parametrize(
-    "work_item__status,case__status",
-    [(models.WorkItem.STATUS_READY, models.Case.STATUS_COMPLETED)],
+    "work_item__status,work_item__child_case,case__status",
+    [(models.WorkItem.STATUS_READY, None, models.Case.STATUS_RUNNING)],
 )
 def test_complete_work_item_last_api(db, work_item, admin_user):
     api.complete_work_item(work_item, admin_user)
@@ -1004,8 +1112,8 @@ def test_complete_work_item_last_api(db, work_item, admin_user):
 
 @pytest.mark.parametrize("task__type,task__form", [(models.Task.TYPE_SIMPLE, None)])
 @pytest.mark.parametrize(
-    "work_item__status,case__status",
-    [(models.WorkItem.STATUS_READY, models.Case.STATUS_COMPLETED)],
+    "work_item__status,work_item__child_case,case__status",
+    [(models.WorkItem.STATUS_READY, None, models.Case.STATUS_RUNNING)],
 )
 def test_skip_work_item_last_api(db, work_item, admin_user):
     api.skip_work_item(work_item, admin_user)
@@ -1016,6 +1124,23 @@ def test_skip_work_item_last_api(db, work_item, admin_user):
     assert work_item.case.status == models.Case.STATUS_COMPLETED
 
 
+@pytest.mark.parametrize("task__type,task__form", [(models.Task.TYPE_SIMPLE, None)])
+@pytest.mark.parametrize(
+    "work_item__status,case__status",
+    [(models.WorkItem.STATUS_READY, models.Case.STATUS_RUNNING)],
+)
+def test_skip_work_item_child_case(db, work_item, case_factory, admin_user):
+    work_item.child_case = case_factory()
+
+    assert work_item.child_case.status == models.Case.STATUS_RUNNING
+
+    api.skip_work_item(work_item, admin_user)
+
+    work_item.refresh_from_db()
+
+    assert work_item.child_case.status == models.Case.STATUS_CANCELED
+
+
 @pytest.mark.parametrize(
     "work_item__status,work_item__child_case,task__type",
     [(models.WorkItem.STATUS_COMPLETED, None, models.Task.TYPE_SIMPLE)],
@@ -1108,31 +1233,141 @@ def test_complete_work_item_same_task_multiple_workflows(
 
 @pytest.mark.parametrize("use_graphql_api", [True, False])
 @pytest.mark.parametrize(
-    "work_item__status,success,expected_status",
+    "action,work_item__status,child_case_status,success,expected_status,expected_child_case_status",
     [
-        (models.WorkItem.STATUS_READY, True, models.WorkItem.STATUS_CANCELED),
-        (models.WorkItem.STATUS_SKIPPED, False, models.WorkItem.STATUS_SKIPPED),
+        (
+            "Cancel",
+            models.WorkItem.STATUS_READY,
+            None,
+            True,
+            models.WorkItem.STATUS_CANCELED,
+            None,
+        ),
+        (
+            "Cancel",
+            models.WorkItem.STATUS_SUSPENDED,
+            None,
+            True,
+            models.WorkItem.STATUS_CANCELED,
+            None,
+        ),
+        (
+            "Cancel",
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+            True,
+            models.WorkItem.STATUS_CANCELED,
+            models.Case.STATUS_CANCELED,
+        ),
+        (
+            "Cancel",
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_SUSPENDED,
+            True,
+            models.WorkItem.STATUS_CANCELED,
+            models.Case.STATUS_CANCELED,
+        ),
+        (
+            "Cancel",
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+            False,
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+        ),
+        (
+            "Suspend",
+            models.WorkItem.STATUS_READY,
+            None,
+            True,
+            models.WorkItem.STATUS_SUSPENDED,
+            None,
+        ),
+        (
+            "Suspend",
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+            True,
+            models.WorkItem.STATUS_SUSPENDED,
+            models.Case.STATUS_SUSPENDED,
+        ),
+        (
+            "Suspend",
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+            False,
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+        ),
+        (
+            "Resume",
+            models.WorkItem.STATUS_READY,
+            None,
+            False,
+            models.WorkItem.STATUS_READY,
+            None,
+        ),
+        (
+            "Resume",
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+            False,
+            models.WorkItem.STATUS_SKIPPED,
+            None,
+        ),
+        (
+            "Resume",
+            models.WorkItem.STATUS_SUSPENDED,
+            None,
+            True,
+            models.WorkItem.STATUS_READY,
+            None,
+        ),
+        (
+            "Resume",
+            models.WorkItem.STATUS_SUSPENDED,
+            models.Case.STATUS_SUSPENDED,
+            True,
+            models.WorkItem.STATUS_READY,
+            models.Case.STATUS_RUNNING,
+        ),
     ],
 )
-def test_cancel_work_item(
+def test_cancel_suspend_resume_work_item(
     db,
     work_item,
     schema_executor,
     admin_user,
+    case_factory,
     use_graphql_api,
+    action,
+    child_case_status,
     success,
     expected_status,
+    expected_child_case_status,
 ):
-    error_msg = "Only READY work items can be cancelled"
+    work_item.child_case = child_case_status and case_factory(status=child_case_status)
+    work_item.save()
+
+    error_msg = {
+        "Cancel": "Only ready or suspended work items can be canceled.",
+        "Suspend": "Only ready work items can be suspended.",
+        "Resume": "Only suspended work items can be resumed.",
+    }[action]
+    _action = action.lower()
 
     if use_graphql_api:
         query = """
-            mutation CancelWorkItem($input: CancelWorkItemInput!) {
-                cancelWorkItem(input: $input) {
+            mutation %sWorkItem($input: %sWorkItemInput!) {
+                %sWorkItem(input: $input) {
                     clientMutationId
                 }
             }
-        """
+        """ % (
+            action,
+            action,
+            _action,
+        )
 
         result = schema_executor(query, variable_values={"input": {"id": work_item.pk}})
 
@@ -1141,14 +1376,19 @@ def test_cancel_work_item(
         else:
             assert error_msg in str(result.errors[0])
     else:
+        fn = getattr(api, f"{_action}_work_item")
+
         if success:
-            api.cancel_work_item(work_item, admin_user)
+            fn(work_item, admin_user)
         else:
             with pytest.raises(ValidationError) as error:
-                api.cancel_work_item(work_item, admin_user)
+                fn(work_item, admin_user)
 
             assert error_msg in str(error.value)
 
     work_item.refresh_from_db()
 
     assert work_item.status == expected_status
+
+    if expected_child_case_status:
+        assert work_item.child_case.status == expected_child_case_status
